Package: later
Type: Package
Title: Utilities for Delaying Function Execution
Version: 0.7.3
Authors@R: c(
    person("Joe", "Cheng", role = c("aut", "cre"), email = "joe@rstudio.com"),
    person(family = "RStudio", role = "cph"),
    person("Marcus", "Geelnard", role = c("ctb", "cph"), comment = "TinyCThread library, https://tinycthread.github.io/"),
    person("Evan", "Nemerson", role = c("ctb", "cph"), comment = "TinyCThread library, https://tinycthread.github.io/")
    )
Description: Executes arbitrary R or C functions some time after the current
    time, after the R execution stack has emptied.
URL: https://github.com/r-lib/later
BugReports: https://github.com/r-lib/later/issues
License: GPL (>= 2)
Imports: Rcpp (>= 0.12.9), rlang
LinkingTo: Rcpp, BH
RoxygenNote: 6.0.1
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2018-06-08 17:43:59 UTC; jcheng
Author: Joe Cheng [aut, cre],
  RStudio [cph],
  Marcus Geelnard [ctb, cph] (TinyCThread library,
    https://tinycthread.github.io/),
  Evan Nemerson [ctb, cph] (TinyCThread library,
    https://tinycthread.github.io/)
Maintainer: Joe Cheng <joe@rstudio.com>
Repository: CRAN
Date/Publication: 2018-06-08 21:42:57 UTC
Built: R 3.4.1; x86_64-w64-mingw32; 2018-08-31 08:07:52 UTC; windows
Archs: x64
